var _widget = { // Namespace for global properties.
    aboutBox: null,
    content: null,
    control: null,
    drawer: null,
    infoBox: null,
    locales: { "Canada": "http://maps.google.ca/", "UK": "http://maps.google.co.uk/", "US": "http://maps.google.com/" },
    map: null,
    mapRequest: null,
    query: null,
    resizedOnce: false,
    searchBox: null,
    url: null
};

function setup() {
    var front = document.getElementById("front");
    front.style.display = "block";
    front.addEventListener("mouseout", mouseExit, false);
    front.addEventListener("mousemove", mouseMove, false);

    var flip = document.getElementById("flip");
    flip.addEventListener("click", showBack, false);
    flip.addEventListener("mouseout", outFlip, false);
    flip.addEventListener("mouseover", overFlip, false);

    var okButtonInfo = document.getElementById("ok-button-info");
    createGenericButton(okButtonInfo, getLocalizedString("OK"), function() { _widget.infoBox.activate() });

    var okButtonAbout = document.getElementById("ok-button-about");
    createGenericButton(okButtonAbout, getLocalizedString("OK"), function() { _widget.aboutBox.activate() });

    var aboutButton = document.getElementById("about-button");
    createGenericButton(aboutButton, getLocalizedString("About"), function() { _widget.aboutBox.activate() });

    var doneButton = document.getElementById("done-button");
    createGenericButton(doneButton, getLocalizedString("Done"), done);

    var resize = document.getElementById("resize");
    resize.addEventListener("mousedown", resizeDown, false);

    var drawerClose = document.getElementById("drawer-close");
    drawerClose.addEventListener("click", function() { _widget.drawer.activate() }, false);

    var top = document.getElementById("top");
    top.addEventListener("click", function() { if (!_widget.content.open) resizeAccordingToPref(true) }, false);

    var shimClosed = document.getElementById("shim-closed");
    shimClosed.addEventListener("click", function() { if (!_widget.content.open) resizeAccordingToPref(true) }, false);

    _widget.aboutBox = new Drawer("about-box", 0.3, Drawer.TOP);
    _widget.infoBox = new Drawer("info-box", 0.5, Drawer.BOTTOM);
    _widget.content = new Drawer("content", 0.5, Drawer.BOTTOM, null, null, resizeWidget);
    _widget.drawer = new Drawer("drawer", 0.7, Drawer.RIGHT, null, resetDrawer, function() { if (window.widget) window.resizeBy(this.by, 0) });
    _widget.searchBox = document.getElementById("search-box");
    _widget.searchBox.keypressed = false;
    _widget.searchBox.setAttribute("placeholder", getLocalizedString("Location"));
    _widget.searchBox.addEventListener("search", prepareForSearch, false);
    _widget.searchBox.addEventListener("keyup", function () { if (_widget.searchBox.value == "") resetMap(false) }, false);
    _widget.searchBox.addEventListener("keypress", function() { _widget.searchBox.keypressed = true }, false);

    var mapControlSelect = document.getElementById("map-control-select");
    mapControlSelect.options[0] = new Option(getLocalizedString("Large"), "Large");
    mapControlSelect.options[1] = new Option(getLocalizedString("Small"), "Small");
    mapControlSelect.options[2] = new Option(getLocalizedString("None"), "None");
    mapControlSelect.addEventListener("change", setMapControl, false);
    setSelected(mapControlSelect, widget.preferenceForKey(createKey("mapControl")));

    var recentSearchesSelect = document.getElementById("recent-searches-select");
    recentSearchesSelect.options[0] = new Option("5 " + getLocalizedString("locations"), "5");
    recentSearchesSelect.options[1] = new Option("10 " + getLocalizedString("locations"), "10");
    recentSearchesSelect.options[2] = new Option("15 " + getLocalizedString("locations"), "15");
    recentSearchesSelect.addEventListener("change", setRecentSearches, false);
    setSelected(recentSearchesSelect, widget.preferenceForKey(createKey("recentSearches")));
    setRecentSearches();

    var localeSelect = document.getElementById("locale-select");
    var i = 0;
    for (var locale in _widget.locales) {
        var url = _widget.locales[locale];
        if (typeof url == "string" && url.indexOf("http://") == 0) {
            localeSelect.options[i++] = new Option(getLocalizedString(locale), locale);
        }
    }
    localeSelect.addEventListener("change", setLocale, false);
    setSelected(localeSelect, widget.preferenceForKey(createKey("locale")) || "US");
    setLocale();

    var panContinuousCheckbox = document.getElementById("pan-continuous-checkbox");
    panContinuousCheckbox.addEventListener("change", setPanContinuous, false);
    panContinuousCheckbox.checked = widget.preferenceForKey(createKey("panContinuous")) ? true : false;

    var panContinuousLabel = document.getElementById("pan-continuous-label");
    panContinuousLabel.addEventListener("click", function() { panContinuousCheckbox.click() }, false);

    document.getElementById("map-control-label").innerText = getLocalizedString("Map control");
    document.getElementById("recent-searches-label").innerText = getLocalizedString("Recent searches");
    document.getElementById("locale-label").innerText = getLocalizedString("Locale");
    document.getElementById("kbd-label").innerText = getLocalizedString("Keyboard shortcuts");
    document.getElementById("zoom-label").innerText = getLocalizedString("Zoom:");
    document.getElementById("recenter-label").innerText = getLocalizedString("Recenter:");
    document.getElementById("view-browser-label").innerText = getLocalizedString("View in browser") + ":";
    document.getElementById("switch-view-label").innerText = getLocalizedString("Switch view:");
    document.getElementById("pan-label").innerText = getLocalizedString("Pan:");
    document.getElementById("pan-continuous-label").innerText = getLocalizedString("Use continuous pan");
    document.getElementById("programming-label").innerText = getLocalizedString("Programming:") + " David Lindquist";
    document.getElementById("design-label").innerText = getLocalizedString("Design:") + " Dave Woodruff";
    document.getElementById("localization-label").innerText = getLocalizedString("Localization:") + " Didier Oswald (Fr), Lorenzo dell'Uva (It)";
    document.getElementById("disclaimer").innerHTML= getLocalizedString("Disclaimer");
    document.getElementById("feedback").innerHTML = getLocalizedString("Feedback");

    document.addEventListener("keypress", onKeyPress, false);
    document.addEventListener("keydown", onKeyDown, false);
    document.addEventListener("keyup", onKeyUp, false);

    window.onblur = onBlur;
    if (window.widget) widget.onremove = onRemove;

    document.getElementById("map").ondblclick = onDoubleClick; // addEventListener doesn't seem
                                                               // to work with dblclick

    _mZoomIn = getLocalizedString("Zoom In");
    _mZoomOut = getLocalizedString("Zoom Out");
    _mZoomSet = getLocalizedString("Click to set zoom level");
    _mZoomDrag = getLocalizedString("Drag to zoom");
    _mPanWest = getLocalizedString("Go left");
    _mPanEast = getLocalizedString("Go right");
    _mPanNorth = getLocalizedString("Go up");
    _mPanSouth = getLocalizedString("Go down");
    _mLastResult = getLocalizedString("Return to the last result");
    _mNormalMap = getLocalizedString("Map");
    _mKeyholeMap = getLocalizedString("Satellite");

    performRequest("Info.plist", outputVersion);

    //_widget.addressBook = AddressBook.recordsAsABigStringOfHypertext("submitQuery", "address-entry", "person", "company", "me", null);

    Scroller.init(document.getElementById("scroller-track"),
                  document.getElementById("scroller-thumb"),
                  document.getElementById("drawer-viewport"),
                  true);
}

window.addEventListener("load", setup, false);

function onBlur() {
    if (_widget.searchBox.value == "") {
        _widget.searchBox.keypressed = false;
        prepareForSearch();
    }
}

function onRemove() {
    if (window.widget) {
        widget.setPreferenceForKey(null, createKey("mapControl"));
        widget.setPreferenceForKey(null, createKey("recentSearches"));
        widget.setPreferenceForKey(null, createKey("locale"));
        widget.setPreferenceForKey(null, createKey("panContinuous"));
        widget.setPreferenceForKey(null, createKey("size"));
    }
}

function onKeyPress(event) {
    if (ignoreKeyEvent(event)) {
        return;
    }
    switch (event.keyCode) {
    case 37: // left arrow
    case 38: // up arrow
    case 39: // right arrow
    case 40: // down arrow
    case 107: // +
    case 187: // +
    case 109: // -
    case 189: // -
        event.cancelBubble = true;
        event.preventDefault();
        event.stopPropagation();
        break;
    }
}

function onKeyUp(event) {
    if (ignoreKeyEvent(event)) {
        return;
    }
    switch (event.keyCode) {
    case 37: // left arrow
    case 38: // up arrow
    case 39: // right arrow
    case 40: // down arrow
        _widget.map.panKeys.remove(event.keyCode);
        break;
    }
}

function onKeyDown(event) {
    if (ignoreKeyEvent(event)) {
        return;
    }
    switch (event.keyCode) {
    case 37: // left arrow
    case 38: // up arrow
    case 39: // right arrow
    case 40: // down arrow
        panMap(event);
        break;
    case 50: // @
        recenterMap(event);
        break;
    case 54: // ^
        viewMapInBrowser();
        break;
    case 107: // +
    case 187: // +
        zoomMap(event, -1);
        break;
    case 109: // -
    case 189: // -
        zoomMap(event, +1);
        break;
    case 192: // ~
        switchMap();
        break;
    }
}

function onDoubleClick(event) {
    if (event.target.tagName == "AREA" ||
        (event.target.src && event.target.src.match(/\.png$/) != null))
        return;
    var map = _widget.map;
    if (map && typeof map.onDoubleClick == "function") {
        map.onDoubleClick(event);
    }
}

function ignoreKeyEvent(event) {
    var front = document.getElementById("front");
    if (_widget.searchBox != event.target &&
        _widget.map &&
        _widget.content.open &&
        front.style.display == "block")
    {
        return false;
    }
    return true;
}

function panMap(event) {
    var map = _widget.map;
    if (map) {
        if (window.widget && widget.preferenceForKey(createKey("panContinuous"))) {
            map.panKeys.add(event.keyCode);
            map.startContinuousPan();
            event.cancelBubble = true;
            event.preventDefault();
            event.stopPropagation();
            return;
        }

        switch (event.keyCode) {
        case 37:
            map.pan(Math.floor(map.viewSize.width * 0.5), 0);
            break;
        case 38:
            map.pan(0, Math.floor(map.viewSize.height * 0.5));
            break;
        case 39:
            map.pan(-Math.floor(map.viewSize.width * 0.5), 0);
            break;
        case 40:
            map.pan(0, -Math.floor(map.viewSize.height * 0.5));
            break;
        }
    }
}

function zoomMap(event, by) {
    if (_widget.map) {
        _widget.map.zoomTo(_widget.map.zoomLevel + by);
    }
}

function recenterMap(event) {
    var map = _widget.map;
    if (map) {
        if (map.lastPageCenter) {
            if (map.lastPageZoom != map.zoomLevel) {
                map.centerAndZoom(map.lastPageCenter, map.lastPageZoom);
            } else {
                map.recenterOrPanToLatLng(b.lastPageCenter);
            }
        } else if (map.lastLatLng) {
            map.recenterOrPanToLatLng(map.lastLatLng);
        }
    }
}

function switchMap(event) {
    var map = _widget.map;
    if (map.spec == map.mapTypes[0]) {
        map.switchSpecification(map.mapTypes[1]);
    } else {
        map.switchSpecification(map.mapTypes[0]);
    }
}

function setSelected(select, value) {
    var option, i = 0;
    while ((option = select.getElementsByTagName("option")[i++])) {
        if (option.value == value) {
            select.selectedIndex = i - 1;
            break;
        }
    }
}

function prepareForSearch() {
    var searchBox = _widget.searchBox;
    var infoBox = _widget.infoBox;
    var content = _widget.content;
    var value = searchBox.value;

    content.onend = null;
    if (value == "") {
        if (content.open && !searchBox.keypressed) {
            collapseWidget();
        }
    } else {
        resetMap(true);
        if (content.open) {
            if (infoBox.open) {
                infoBox.onend = function() {
                    doSearch(value);
                    searchBox.keypressed = false;
                    this.onend = null;
                }
                infoBox.activate();
            } else {
                doSearch(value);
                searchBox.keypressed = false;
            }
        } else {
            resizeAccordingToPref(true, value);
        }
    }
}

function performRequest(url, callback) {
    var request = new XMLHttpRequest();
    request.overrideMimeType("text/xml");
    request.setRequestHeader("Cache-Control", "no-cache");
    request.setRequestHeader("Accept-Language", getLocalizedString("Language") );
    request.open("GET", url);
    request.onload = function(e) { callback(e, request) }
    request.send(null);
    return request;
}

function doSearch(value) {
    _widget.query = normalizeQuery(value);
    if (_widget.mapRequest) {
        _widget.mapRequest.abort();
    }
    _widget.mapRequest = performRequest(_widget.url + "maps?q=" + _widget.query + "&hl=" + getLocalizedString("Language"),  parseResponse);
}

function normalizeQuery(query) {
    var q = query;

    // only submit first 5 numbers in ZIP+4 code
    q = q.replace(/(\b\d{5})-\d{4}\b/, "$1");

    // url encode for good measure
    q = encodeURIComponent(q);

    return q;
}

function parseResponse(event, request) {
    try {
        var response = request.responseText;
        var match;
        if ((match = response.match(/<\?xml version="1.0"\?><page>(.*?)<\/page>/)) != null) {
            var xml = match[0].replace(/=\"\//g, "=\"" + _widget.url);
            loadMap(xml);

            var error, refinements;
            if ((error = xml.match(/<error>(.*?)<\/error>/)) != null) {
                showInfo(error[1]);
            } else if ((refinements = xml.match(/<refinements>(.*?)<\/refinements>/)) != null) {
                showRefinements(refinements[1]);
            }
        }
    } catch (ex) {
        document.getElementById("status").innerText = getLocalizedString("Data unavailable.");
    }
    _widget.mapRequest = null;
}

function getVersion(event, request) {
    var version = "?";
    try {
        var response = request.responseXML;
        var key, i = 0;
        while ((key = response.getElementsByTagName("key")[i++])) {
            if (key.firstChild.data == "CFBundleVersion") break;
        }
        if (key) {
            var string = key.nextSibling;
            while (string && string.nodeType != 1) {
                string = string.nextSibling;
            }
            version = string ? string.firstChild.data : "?";
        }
    } catch (ex) {}
    return version;
}

function outputVersion(event, request) {
    var version = document.getElementById("version");
    version.innerText = "v" + getVersion(event, request);
}

function loadMap(xml) {
    var container = document.getElementById("map");
    if (_widget.map) {
        _widget.map.deleteTiles();
    }
    _widget.map = new _Map(container, null, null, null, 1);
    _widget.map.loadVPageStr(xml);

    if (window.widget) {
        var mapControl = widget.preferenceForKey(createKey("mapControl"));
        switch (mapControl) {
        case "None":
            break;
        case "Small":
            _widget.map.showSmallMapControl();
            _widget.control = _widget.map.container.lastChild;
            break;
        default:
            _widget.map.showMapControl();
            _widget.control = _widget.map.container.lastChild;
            break;
        }
    }

    document.getElementById("status").style.visibility = "hidden";
    document.getElementById("content-spinner").style.visibility = "hidden";

    createLinks();
    fixMarkers(); // blech
}

function createLinks() {
    var map, sat, view, adbook;
    var blinks = document.getElementById("bottom-links");
    blinks.innerHTML = "";

    if (_widget.map) {
        _widget.map.createSpecToggleLinks(blinks, true);
        view = document.createElement("a");
        view.setAttribute("href", "javascript:void(0)");
        view.addEventListener("click", viewMapInBrowser, false);
    } else {
        map = document.createElement("span");
        sat = document.createElement("span");
        view = document.createElement("span");
        map.appendChild(document.createTextNode(getLocalizedString("Map")));
        sat.appendChild(document.createTextNode(getLocalizedString("Satellite")));
        blinks.appendChild(map);
        blinks.appendChild(document.createTextNode(" - "));
        blinks.appendChild(sat);
    }
    view.appendChild(document.createTextNode(getLocalizedString("View in browser")));

    adbook = document.createElement("a");
    adbook.setAttribute("href", "javascript:void(0)");
    adbook.addEventListener("click", activateAddressBook, false);
    adbook.appendChild(document.createTextNode(getLocalizedString("Address Book")));

    blinks.appendChild(document.createTextNode(" - "));
    blinks.appendChild(view);
    blinks.appendChild(document.createTextNode(" - "));
    blinks.appendChild(adbook);
}

function showInfo(html) {
    document.getElementById("info").innerHTML = html.replace(new RegExp("\\\\'", "g"), "'");
    document.getElementById("info-box").style.height = document.getElementById("info").clientHeight + "px";

    if (_widget.infoBox.open == false) {
        _widget.infoBox.activate();
    }
}

function showRefinements(str) {
    var html = "<p><b>" + getLocalizedString("Did you mean:") + "</b></p><ul>";

    var pattern = /<refinement>(.*?)<\/refinement>/g;
    var refinement;
    while ((refinement = pattern.exec(str)) != null) {
        var query = refinement[0].match(/<query>(.*?)<\/query>/);
        var description = refinement[0].match(/<description>(.*?)<\/description>/);
        if (query != null && description != null) {
            html += '<li><a href="javascript:submitQuery(\'' + query[1] + '\')">' + description[1] + '</a></li>';
        }
    }
    html += "</ul>";

    showInfo(html);
}

function populateDrawer(title, html) {
    document.getElementById("drawer-header").innerText = title;
    document.getElementById("drawer-content").innerHTML = html;
    Scroller.update();
    //if (_widget.drawer.open == false) {
    //    _widget.drawer.activate();
    //}
}

function resetDrawer() {
    populateDrawer("", "");
    document.getElementById("drawer-spinner").style.visibility = "visible";
}

function submitQuery(query) {
    _widget.searchBox.value = query;
    prepareForSearch();
}

function resetMap(loading) {
    _widget.control = null;
    _widget.map = null;
    _widget.query = null;
    if (loading) {
        //document.getElementById("status").innerText = getLocalizedString("Loading...");
        //document.getElementById("status").style.visibility = "visible";
        document.getElementById("content-spinner").style.visibility = "visible";
    }
    document.getElementById("map").innerHTML = "";
    document.getElementById("info").innerHTML = "";

    createLinks();
}

function fixMarkers() {
    var pattern = /(.*)ff\.gif$/;
    var img, i = 0;
    while ((img = document.getElementsByTagName("img")[i++])) {
        if (pattern.test(img.src)) {
            img.src = img.src.replace(pattern, "$1.png");
        }
    }
}

function viewMapInBrowser() {
    if (window.widget) {
        var map = _widget.map;
        var query = _widget.query.replace(/%20/g, "+"); // widget.openURL does not seem to like %20
        var ll = map.getCenterLatLng();
        var spn = map.getSpanLatLng();
        var t = map.spec.getURLArg();
        var url = _widget.url + "maps?q=" + decodeURIComponent(query) + "&ll=" + ll.y + "," + ll.x + "&spn=" + spn.height + "," + spn.width + (t ? "&t=" + t : "");
        widget.openURL(url);
    }
}

function activateAddressBook() {
    if (!_widget.drawer.open) {
        _widget.drawer.onend = function() {
            setTimeout("showAddressBook()", 1000);
            this.onend = null;
        }
    }
    _widget.drawer.activate();
}

function showAddressBook() {
    populateDrawer(getLocalizedString("Address Book"), AddressBook.recordsAsABigStringOfHypertext("submitQuery", "address-entry", "person", "company", "me", null));
    document.getElementById("drawer-spinner").style.visibility = "hidden";
}

function resizeWidget() {
    if (window.widget) {
        window.resizeBy(0, this.by);

        var shim = document.getElementById("shim-opened");
        if (this.open && this.clip[Drawer.BOTTOM] < 30 && shim.style.display != "none") {
            shim.style.display = "none";
        }
    }
}

function expandWidget(value) {
    if (_widget.content.open || (!_widget.content.open && _widget.content.moving))
        return;

    document.getElementById("shim-opened").style.display = "block";
    document.getElementById("shim-closed").style.display = "none";

    if (value)
        _widget.content.onend = function() { doSearch(value) };
    else
        createLinks();

    _widget.content.activate();
    _widget.searchBox.keypressed = false;
}

function collapseWidget() {
    var drawer = _widget.drawer;
    var infoBox = _widget.infoBox;
    var content = _widget.content;

    if (!content.open)
        return;

    if (drawer.open) {
        drawer.onend = function() {
            collapseWidget();
            this.onend = null;
        }
        drawer.activate();
    } else if (infoBox.open) {
        infoBox.onend = function() {
            collapseWidget();
            this.onend = null;
        }
        infoBox.activate();
    } else if (content.open) {
        document.getElementById("shim-closed").style.display = "block";
        resetMap();
        content.onend = function() {
            resizeAccordingToPref(false);
            this.onend = null;
        }
        content.activate();
    }
}

var resizing = false;

function resizeAccordingToPref(opening, query) {
    if (resizing || !window.widget)
        return;

    var size = widget.preferenceForKey(createKey("size"));
    var x, y;

    if (opening) {
        x = (size) ? Number(size.split(",")[0]) : 424;
        y = (size) ? Number(size.split(",")[1]) : 401;

        if (!_widget.resizedOnce) {
            resizeComponenetsBy(null, y - 401);
            _widget.resizedOnce = true;
        }

        if (query) {
            animateWidthResize(x, opening, query);
        } else {
            animateWidthResize(x, opening);
        }
    } else {
        animateWidthResize(424);
    }
}

function animateWidthResize(x, opening, query) {
    var dir = ((x - window.innerWidth) < 0) ? -1 : 1;
    if (x != window.innerWidth) {
        var deltaX = Math.round((x - window.innerWidth) * 0.5) || 1;
        resizeComponenetsBy(dir * Math.abs(deltaX));
        window.resizeBy(dir * Math.abs(deltaX), 0);

        if (query)
            setTimeout("animateWidthResize(" + x + ", " + opening + ", '" + query + "')", 10);
        else
            setTimeout("animateWidthResize(" + x + ", " + opening + ")", 10);

        resizing = true;
        return;
    }

    resizing = false;

    if (opening) {
        expandWidget(query);
    }
}

function setMapControl() {
    var select = document.getElementById("map-control-select");
    var mapControl = select.options[select.selectedIndex].value;
    if (_widget.map) {
        if (_widget.control) {
            _widget.map.container.removeChild(_widget.control);
        }
        switch (mapControl) {
        case "None":
            _widget.control = null;
            break;
        case "Small":
            _widget.map.showSmallMapControl();
            _widget.control = _widget.map.container.lastChild;
            break;
        default:
            _widget.map.showMapControl();
            _widget.control = _widget.map.container.lastChild;
            break;
        }
    }
    if (window.widget) {
        widget.setPreferenceForKey(mapControl, createKey("mapControl"));
    }
}

function setRecentSearches() {
    var select = document.getElementById("recent-searches-select");
    var recentSearches = select.options[select.selectedIndex].value;

    if (_widget.searchBox) {
        _widget.searchBox.setAttribute("results", recentSearches);
    }

    if (window.widget) {
        widget.setPreferenceForKey(recentSearches, createKey("recentSearches"));
    }
}

function setLocale() {
    var select = document.getElementById("locale-select");
    var locale = select.options[select.selectedIndex].value;

    _widget.url = _widget.locales[locale];

    if (window.widget) {
        widget.setPreferenceForKey(locale, createKey("locale"));
    }
}

function setPanContinuous() {
    var checkbox = document.getElementById("pan-continuous-checkbox");

    if (window.widget) {
        widget.setPreferenceForKey(checkbox.checked || null, createKey("panContinuous"));
    }
}

function done() {
    var about = _widget.aboutBox;

    if (about.open) {
        about.onend = function() {
            showFront();
            this.onend = null;
        }
        about.activate();
    } else {
        showFront();
    }
}

function showBack() {
    var back = document.getElementById("back");
    var front = document.getElementById("front");

    if (window.widget) {
        widget.prepareForTransition("ToBack");
    }

    back.style.display = "block";
    front.style.display = "none";

    if (window.widget) {
        setTimeout ("widget.performTransition();", 0);
    }

    outFlip(); // reset the fliprollie
}

function showFront() {
    var back = document.getElementById("back");
    var front = document.getElementById("front");

    if (window.widget) {
        widget.prepareForTransition("ToFront");
    }

    back.style.display = "none";
    front.style.display = "block";

    if (window.widget) {
        setTimeout ("widget.performTransition();", 0);
    }
}

function overFlip(event) {
    document.getElementById("fliprollie").style.display = "block";
}

function outFlip(event) {
    document.getElementById("fliprollie").style.display = "none";
}

var growboxInset;
var currentSize;

function resizeDown(event) {
    document.addEventListener("mousemove", resizeMove, true);
    document.addEventListener("mouseup", resizeUp, true);

    growboxInset = { x: (window.innerWidth - event.x), y: (window.innerHeight - event.y) };
    currentSize = { x: window.innerWidth, y: window.innerHeight };

    event.stopPropagation();
    event.preventDefault();
}

function resizeMove(event) {
    var x = event.x + growboxInset.x;
    var y = event.y + growboxInset.y;

    var minX = 424 + (_widget.drawer.open ? 225 : 0);
    var minY = 401;

    if (x < minX) x = minX;
    if (y < minY) y = minY;

    var deltaX = x - currentSize.x;
    var deltaY = y - currentSize.y;

    window.resizeTo(x, y);
    resizeComponenetsBy(deltaX, deltaY);

    Scroller.update();

    if (_widget.map)
        _widget.map.onResize();

    currentSize.x = x;
    currentSize.y = y;

    event.stopPropagation();
    event.preventDefault();
}

function resizeUp(event) {
    document.removeEventListener("mousemove", resizeMove, true);
    document.removeEventListener("mouseup", resizeUp, true);

    var size = [currentSize.x, currentSize.y].join(",");
    widget.setPreferenceForKey(size, createKey("size"));

    event.stopPropagation();
    event.preventDefault();
}

function resizeComponenetsBy(deltaX, deltaY) {
    if (deltaX) {
        var front = document.getElementById("front");
        front.style.width = front.clientWidth + deltaX + "px";
    }
    if (deltaY) {
        var content = document.getElementById("content");
        var map = document.getElementById("map");
        var drawer = document.getElementById("drawer");
        content.style.height = content.clientHeight + deltaY + "px";
        map.style.height = map.clientHeight + deltaY + "px";
        drawer.style.height = drawer.clientHeight + deltaY + "px";
    }
}

var flipShown = false;
var animation = { duration: 0, starttime: 0, to: 1.0, now: 0.0, from: 0.0, firstElement: null, timer: null };

function mouseMove(event) {
    if (!flipShown) {
        if (animation.timer != null) {
            clearInterval(animation.timer);
            animation.timer = null;
        }

        var starttime = (new Date).getTime() - 13;

        animation.duration = 500;
        animation.starttime = starttime;
        animation.firstElement = document.getElementById("flip");
        animation.timer = setInterval("animate();", 13);
        animation.from = animation.now;
        animation.to = 1.0;
        animate();
        flipShown = true;
    }
}

function mouseExit(event) {
    if (flipShown) {
        // fade in the info button
        if (animation.timer != null) {
            clearInterval(animation.timer);
            animation.timer = null;
        }

        var starttime = (new Date).getTime() - 13;

        animation.duration = 500;
        animation.starttime = starttime;
        animation.firstElement = document.getElementById("flip");
        animation.timer = setInterval("animate();", 13);
        animation.from = animation.now;
        animation.to = 0.0;
        animate();
        flipShown = false;
    }
}

function animate() {
    var T;
    var ease;
    var time = (new Date).getTime();

    T = limit_3(time-animation.starttime, 0, animation.duration);

    if (T >= animation.duration) {
        clearInterval(animation.timer);
        animation.timer = null;
        animation.now = animation.to;
    } else {
        ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
        animation.now = computeNextFloat(animation.from, animation.to, ease);
    }

    animation.firstElement.style.opacity = animation.now;
}

function limit_3 (a, b, c) {
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease) {
    return from + (to - from) * ease;
}

function getLocalizedString(key) {
    if (typeof localizedStrings != "undefined" &&
        typeof localizedStrings[key] != "undefined")
    {
        return localizedStrings[key];
    }
    return key;
}

function createKey(key) {
    if (window.widget) {
        return widget.identifier + "-" + key;
    }
    return key;
}
